@@ -85,9 +85,9 @@ MarkedBlock::Handle::~Handle()
 
 MarkedBlock::MarkedBlock(VM& vm, Handle& handle)
     : m_needsDestruction(handle.needsDestruction())
+    , m_version(vm.heap.objectSpace().version())
     , m_handle(handle)
     , m_vm(&vm)
-    , m_version(vm.heap.objectSpace().version())
 {
     unsigned cellsPerBlock = MarkedSpace::blockPayload / handle.cellSize();
     double markCountBias = -(Options::minMarkedBlockUtilization() * cellsPerBlock);
@@ -148,7 +148,7 @@ FreeList MarkedBlock::Handle::specializedSweep()
             static_cast<JSCell*>(cell)->callDestructor(*vm());
 
         if (sweepMode == SweepToFreeList) {
-            FreeCell* freeCell = reinterpret_cast<FreeCell*>(cell);
+            FreeCell* freeCell = reinterpret_cast_ptr<FreeCell*>(cell);
             if (scribbleMode == Scribble)
                 scribble(freeCell, cellSize());
             freeCell->next = head;
@@ -357,14 +357,14 @@ void MarkedBlock::Handle::flipIfNecessary()
 
 void MarkedBlock::flipIfNecessarySlow()
 {
-    ASSERT(m_version != vm()->heap.objectSpace().version());
+    ASSERT(needsFlip());
     clearMarks();
 }
 
 void MarkedBlock::flipIfNecessaryConcurrentlySlow()
 {
     LockHolder locker(m_lock);
-    if (m_version != vm()->heap.objectSpace().version())
+    if (needsFlip())
         clearMarks();
 }
 
@@ -388,7 +388,7 @@ void MarkedBlock::assertFlipped()
 
 bool MarkedBlock::needsFlip()
 {
-    return vm()->heap.objectSpace().version() != m_version;
+    return needsFlip(vm()->heap.objectSpace().version());
 }
 
 bool MarkedBlock::Handle::needsFlip()
