@@ -26,6 +26,7 @@
 #ifndef JSArrayBufferView_h
 #define JSArrayBufferView_h
 
+#include "CopyBarrier.h"
 #include "JSObject.h"
 
 namespace JSC {
@@ -53,7 +54,7 @@ class LLIntOffsetsExtractor;
 // Typed array views have different modes depending on how big they are and
 // whether the user has done anything that requires a separate backing
 // buffer or the DOM-specified neutering capabilities.
-enum TypedArrayMode {
+enum TypedArrayMode : uint32_t {
     // Small and fast typed array. B is unused, V points to a vector
     // allocated in copied space, and M = FastTypedArray. V's liveness is
     // determined entirely by the view's liveness.
@@ -179,6 +180,8 @@ protected:
 
     ArrayBuffer* existingBufferInButterfly();
 
+    static String toStringName(const JSObject*, ExecState*);
+
     CopyBarrier<char> m_vector; // this is really a void*, but void would not work here.
     uint32_t m_length;
     TypedArrayMode m_mode;
