@@ -43,8 +43,13 @@ bool JSValue::strictEqualSlowCase(ExecState* exec, JSValue v1, JSValue v2)
 NEVER_INLINE JSValue jsAddSlowCase(CallFrame* callFrame, JSValue v1, JSValue v2)
 {
     // exception for the Date exception in defaultValue()
+    VM& vm = callFrame->vm();
     JSValue p1 = v1.toPrimitive(callFrame);
+    if (UNLIKELY(vm.exception()))
+        return JSValue();
     JSValue p2 = v2.toPrimitive(callFrame);
+    if (UNLIKELY(vm.exception()))
+        return JSValue();
 
     if (p1.isString())
         return jsString(callFrame, asString(p1), p2.toString(callFrame));
