@@ -29,6 +29,14 @@
 
 #include <mach-o/dyld_priv.h>
 
+#if PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED < 100000
+#define DYLD_IOS_VERSION_10_0 0x000A0000
+#endif
+
+#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 101200
+#define DYLD_MACOSX_VERSION_10_12 0x000A0C00
+#endif
+
 #else
 
 #define DYLD_IOS_VERSION_3_0 0x00030000
@@ -37,6 +45,9 @@
 #define DYLD_IOS_VERSION_6_0 0x00060000
 #define DYLD_IOS_VERSION_7_0 0x00070000
 #define DYLD_IOS_VERSION_9_0 0x00090000
+#define DYLD_IOS_VERSION_10_0 0x000A0000
+
+#define DYLD_MACOSX_VERSION_10_12 0x000A0C00
 
 #endif
 
