@@ -45,7 +45,9 @@
 #include "SQLiteStatement.h"
 #include "SQLiteTransaction.h"
 #include "ThreadSafeDataBuffer.h"
+#include <heap/HeapInlines.h>
 #include <heap/StrongInlines.h>
+#include <runtime/AuxiliaryBarrierInlines.h>
 #include <runtime/JSCJSValueInlines.h>
 #include <runtime/JSGlobalObject.h>
 #include <runtime/StructureInlines.h>
@@ -2056,16 +2058,18 @@ void SQLiteIDBBackingStore::deleteBackingStore()
     {
         bool errored = true;
 
-        SQLiteStatement sql(*m_sqliteDB, ASCIILiteral("SELECT fileName FROM BlobFiles;"));
-        if (sql.prepare() == SQLITE_OK) {
-            int result = sql.step();
-            while (result == SQLITE_ROW) {
-                blobFiles.append(sql.getColumnText(0));
-                result = sql.step();
-            }
+        if (m_sqliteDB) {
+            SQLiteStatement sql(*m_sqliteDB, ASCIILiteral("SELECT fileName FROM BlobFiles;"));
+            if (sql.prepare() == SQLITE_OK) {
+                int result = sql.step();
+                while (result == SQLITE_ROW) {
+                    blobFiles.append(sql.getColumnText(0));
+                    result = sql.step();
+                }
 
-            if (result == SQLITE_DONE)
-                errored = false;
+                if (result == SQLITE_DONE)
+                    errored = false;
+            }
         }
 
         if (errored)
