@@ -19,9 +19,9 @@
 #include "config.h"
 #include "WebKitDOMCustom.h"
 
+#include "DOMTokenList.h"
 #include "JSMainThreadExecState.h"
 #include "SerializedScriptValue.h"
-#include "WebKitDOMDOMSettableTokenListPrivate.h"
 #include "WebKitDOMDOMWindowPrivate.h"
 #include "WebKitDOMHTMLInputElement.h"
 #include "WebKitDOMHTMLInputElementPrivate.h"
@@ -86,7 +86,11 @@ gboolean webkit_dom_dom_window_webkit_message_handlers_post_message(WebKitDOMDOM
         return FALSE;
 
     WebCore::JSMainThreadNullState state;
-    handler->postMessage(WebCore::SerializedScriptValue::create(String::fromUTF8(message)));
+    WebCore::ExceptionCode ec = 0;
+    handler->postMessage(WebCore::SerializedScriptValue::create(String::fromUTF8(message)), ec);
+    if (ec)
+        return FALSE;
+
     return TRUE;
 }
 
@@ -95,5 +99,27 @@ void webkit_dom_html_link_element_set_sizes(WebKitDOMHTMLLinkElement* linkElemen
     g_return_if_fail(WEBKIT_DOM_IS_HTML_LINK_ELEMENT(linkElement));
     g_return_if_fail(value);
 
-    core(linkElement)->setSizes(String::fromUTF8(value));
+    core(linkElement)->sizes().setValue(String::fromUTF8(value));
+}
+
+gboolean webkit_dom_html_input_element_get_auto_filled(WebKitDOMHTMLInputElement* self)
+{
+    g_return_val_if_fail(WEBKIT_DOM_IS_HTML_INPUT_ELEMENT(self), FALSE);
+
+    return WebKit::core(self)->isAutoFilled();
+}
+
+void webkit_dom_html_input_element_set_auto_filled(WebKitDOMHTMLInputElement* self, gboolean value)
+{
+    g_return_if_fail(WEBKIT_DOM_IS_HTML_INPUT_ELEMENT(self));
+
+    WebKit::core(self)->setAutoFilled(value);
+}
+
+void webkit_dom_html_input_element_set_editing_value(WebKitDOMHTMLInputElement* self, const gchar* value)
+{
+    g_return_if_fail(WEBKIT_DOM_IS_HTML_INPUT_ELEMENT(self));
+    g_return_if_fail(value);
+
+    WebKit::core(self)->setEditingValue(WTF::String::fromUTF8(value));
 }
