@@ -178,27 +178,27 @@ public:
 
     // Event handlers.
 
-    virtual void handleClickEvent(MouseEvent*);
-    virtual void handleMouseDownEvent(MouseEvent*);
+    virtual void handleClickEvent(MouseEvent&);
+    virtual void handleMouseDownEvent(MouseEvent&);
     virtual void willDispatchClick(InputElementClickState&);
     virtual void didDispatchClick(Event*, const InputElementClickState&);
-    virtual void handleDOMActivateEvent(Event*);
-    virtual void handleKeydownEvent(KeyboardEvent*);
-    virtual void handleKeypressEvent(KeyboardEvent*);
-    virtual void handleKeyupEvent(KeyboardEvent*);
-    virtual void handleBeforeTextInsertedEvent(BeforeTextInsertedEvent*);
-    virtual void forwardEvent(Event*);
+    virtual void handleDOMActivateEvent(Event&);
+    virtual void handleKeydownEvent(KeyboardEvent&);
+    virtual void handleKeypressEvent(KeyboardEvent&);
+    virtual void handleKeyupEvent(KeyboardEvent&);
+    virtual void handleBeforeTextInsertedEvent(BeforeTextInsertedEvent&);
+    virtual void forwardEvent(Event&);
 
 #if ENABLE(TOUCH_EVENTS)
-    virtual void handleTouchEvent(TouchEvent*);
+    virtual void handleTouchEvent(TouchEvent&);
 #endif
 
     // Helpers for event handlers.
 
-    virtual bool shouldSubmitImplicitly(Event*);
+    virtual bool shouldSubmitImplicitly(Event&);
     virtual PassRefPtr<HTMLFormElement> formForSubmission() const;
     virtual bool hasCustomFocusLogic() const;
-    virtual bool isKeyboardFocusable(KeyboardEvent*) const;
+    virtual bool isKeyboardFocusable(KeyboardEvent&) const;
     virtual bool isMouseFocusable() const;
     virtual bool shouldUseInputMethod() const;
     virtual void handleFocusEvent(Node* oldFocusedNode, FocusDirection);
@@ -295,7 +295,7 @@ public:
     virtual unsigned height() const;
     virtual unsigned width() const;
 
-    void dispatchSimulatedClickIfActive(KeyboardEvent*) const;
+    void dispatchSimulatedClickIfActive(KeyboardEvent&) const;
 
 #if ENABLE(DATALIST_ELEMENT)
     virtual void listAttributeTargetChanged();
