@@ -31,6 +31,7 @@
 #ifndef ApplicationCacheHost_h
 #define ApplicationCacheHost_h
 
+#include "PlatformExportMacros.h"
 #include "URL.h"
 #include <wtf/Deque.h>
 #include <wtf/PassRefPtr.h>
@@ -41,10 +42,11 @@ namespace WebCore {
     class DOMApplicationCache;
     class DocumentLoader;
     class Frame;
-    class ResourceLoader;
     class ResourceError;
+    class ResourceLoader;
     class ResourceRequest;
     class ResourceResponse;
+    class SharedBuffer;
     class SubstituteData;
     class ApplicationCache;
     class ApplicationCacheGroup;
@@ -110,6 +112,8 @@ namespace WebCore {
         explicit ApplicationCacheHost(DocumentLoader&);
         ~ApplicationCacheHost();
 
+        static URL createFileURL(const String&);
+
         void selectCacheWithoutManifest();
         void selectCacheWithManifest(const URL& manifestURL);
 
@@ -120,13 +124,13 @@ namespace WebCore {
         void finishedLoadingMainResource();
         void failedLoadingMainResource();
 
-        WEBCORE_EXPORT bool maybeLoadResource(ResourceLoader*, const ResourceRequest&, const URL& originalURL);
+        WEBCORE_EXPORT bool maybeLoadResource(ResourceLoader&, const ResourceRequest&, const URL& originalURL);
         WEBCORE_EXPORT bool maybeLoadFallbackForRedirect(ResourceLoader*, ResourceRequest&, const ResourceResponse&);
         WEBCORE_EXPORT bool maybeLoadFallbackForResponse(ResourceLoader*, const ResourceResponse&);
         WEBCORE_EXPORT bool maybeLoadFallbackForError(ResourceLoader*, const ResourceError&);
 
-        bool maybeLoadSynchronously(ResourceRequest&, ResourceError&, ResourceResponse&, Vector<char>& data);
-        void maybeLoadFallbackSynchronously(const ResourceRequest&, ResourceError&, ResourceResponse&, Vector<char>& data);
+        bool maybeLoadSynchronously(ResourceRequest&, ResourceError&, ResourceResponse&, RefPtr<SharedBuffer>&);
+        void maybeLoadFallbackSynchronously(const ResourceRequest&, ResourceError&, ResourceResponse&, RefPtr<SharedBuffer>&);
 
         bool canCacheInPageCache();
 
@@ -146,7 +150,7 @@ namespace WebCore {
         CacheInfo applicationCacheInfo();
 
         bool shouldLoadResourceFromApplicationCache(const ResourceRequest&, ApplicationCacheResource*&);
-        bool getApplicationCacheFallbackResource(const ResourceRequest&, ApplicationCacheResource*&, ApplicationCache* = 0);
+        bool getApplicationCacheFallbackResource(const ResourceRequest&, ApplicationCacheResource*&, ApplicationCache* = nullptr);
 
     private:
         bool isApplicationCacheEnabled();
@@ -169,7 +173,7 @@ namespace WebCore {
         friend class ApplicationCacheGroup;
         friend class ApplicationCacheStorage;
 
-        bool scheduleLoadFallbackResourceFromApplicationCache(ResourceLoader*, ApplicationCache* = 0);
+        bool scheduleLoadFallbackResourceFromApplicationCache(ResourceLoader*, ApplicationCache* = nullptr);
         void setCandidateApplicationCacheGroup(ApplicationCacheGroup* group);
         ApplicationCacheGroup* candidateApplicationCacheGroup() const { return m_candidateApplicationCacheGroup; }
         void setApplicationCache(PassRefPtr<ApplicationCache> applicationCache);
@@ -177,7 +181,6 @@ namespace WebCore {
         ApplicationCache* mainResourceApplicationCache() const { return m_mainResourceApplicationCache.get(); }
         bool maybeLoadFallbackForMainError(const ResourceRequest&, const ResourceError&);
 
-
         // The application cache that the document loader is associated with (if any).
         RefPtr<ApplicationCache> m_applicationCache;
 
