@@ -24,8 +24,7 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef Extensions3DOpenGLES_h
-#define Extensions3DOpenGLES_h
+#pragma once
 
 #include "Extensions3DOpenGLCommon.h"
 
@@ -63,6 +62,8 @@ namespace WebCore {
 
 class Extensions3DOpenGLES : public Extensions3DOpenGLCommon {
 public:
+    // This class only needs to be instantiated by GraphicsContext3D implementations.
+    explicit Extensions3DOpenGLES(GraphicsContext3D*, bool useIndexedGetString);
     virtual ~Extensions3DOpenGLES();
 
     virtual void framebufferTexture2DMultisampleIMG(unsigned long target, unsigned long attachment, unsigned long textarget, unsigned int texture, int level, unsigned long samples);
@@ -95,10 +96,6 @@ public:
     virtual void getnUniformivEXT(GC3Duint program, int location, GC3Dsizei bufSize, int *params);
 
 protected:
-    // This class only needs to be instantiated by GraphicsContext3D implementations.
-    friend class GraphicsContext3D;
-    Extensions3DOpenGLES(GraphicsContext3D*);
-
     virtual bool supportsExtension(const String&);
     virtual String getExtensions();
 
@@ -128,5 +125,3 @@ protected:
 } // namespace WebCore
 
 #endif // USE(OPENGL_ES_2)
-
-#endif // Extensions3DOpenGLES_h
