@@ -23,15 +23,6 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-WebInspector.JavaScriptRuntimeCompletionProvider = function()
-{
-    WebInspector.Object.call(this);
-
-    console.assert(!WebInspector.JavaScriptRuntimeCompletionProvider._instance);
-
-    WebInspector.debuggerManager.addEventListener(WebInspector.DebuggerManager.Event.ActiveCallFrameDidChange, this._clearLastProperties, this);
-};
-
 Object.defineProperty(WebInspector, "javaScriptRuntimeCompletionProvider",
 {
     get: function()
@@ -42,12 +33,20 @@ Object.defineProperty(WebInspector, "javaScriptRuntimeCompletionProvider",
     }
 });
 
-WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
-    constructor: WebInspector.JavaScriptRuntimeCompletionProvider,
+WebInspector.JavaScriptRuntimeCompletionProvider = class JavaScriptRuntimeCompletionProvider extends WebInspector.Object
+{
+    constructor()
+    {
+        super();
+
+        console.assert(!WebInspector.JavaScriptRuntimeCompletionProvider._instance);
+
+        WebInspector.debuggerManager.addEventListener(WebInspector.DebuggerManager.Event.ActiveCallFrameDidChange, this._clearLastProperties, this);
+    }
 
     // Protected
 
-    completionControllerCompletionsNeeded: function(completionController, defaultCompletions, base, prefix, suffix, forced)
+    completionControllerCompletionsNeeded(completionController, defaultCompletions, base, prefix, suffix, forced)
     {
         // Don't allow non-forced empty prefix completions unless the base is that start of property access.
         if (!forced && !prefix && !/[.[]$/.test(base)) {
@@ -103,8 +102,10 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
         var activeCallFrame = WebInspector.debuggerManager.activeCallFrame;
         if (!base && activeCallFrame && !this._alwaysEvaluateInWindowContext)
             activeCallFrame.collectScopeChainVariableNames(receivedPropertyNames.bind(this));
-        else
-            WebInspector.runtimeManager.evaluateInInspectedWindow(base, "completion", true, true, false, false, evaluated.bind(this));
+        else {
+            let options = {objectGroup: "completion", includeCommandLineAPI: true, doNotPauseOnExceptionsAndMuteConsole: true, returnByValue: false, generatePreview: false, saveResult: false};
+            WebInspector.runtimeManager.evaluateInInspectedWindow(base, options, evaluated.bind(this));
+        }
 
         function updateLastPropertyNames(propertyNames)
         {
@@ -126,7 +127,35 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
                 return;
             }
 
-            function getCompletions(primitiveType)
+            function inspectedPage_evalResult_getArrayCompletions(primitiveType)
+            {
+                var array = this;
+                var arrayLength;
+
+                var resultSet = {};
+                for (var o = array; o; o = o.__proto__) {
+                    try {
+                        if (o === array && o.length) {
+                            // If the array type has a length, don't include a list of all the indexes.
+                            // Include it at the end and the frontend can build the list.
+                            arrayLength = o.length;
+                        } else {
+                            var names = Object.getOwnPropertyNames(o);
+                            for (var i = 0; i < names.length; ++i)
+                                resultSet[names[i]] = true;
+                        }
+                    } catch (e) {
+                        // Ignore
+                    }
+                }
+
+                if (arrayLength)
+                    resultSet["length"] = arrayLength;
+
+                return resultSet;
+            }
+
+            function inspectedPage_evalResult_getCompletions(primitiveType)
             {
                 var object;
                 if (primitiveType === "string")
@@ -135,6 +164,8 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
                     object = new Number(0);
                 else if (primitiveType === "boolean")
                     object = new Boolean(false);
+                else if (primitiveType === "symbol")
+                    object = Symbol();
                 else
                     object = this;
 
@@ -152,11 +183,14 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
                 return resultSet;
             }
 
-            if (result.type === "object" || result.type === "function")
-                result.callFunctionJSON(getCompletions, undefined, receivedPropertyNames.bind(this));
-            else if (result.type === "string" || result.type === "number" || result.type === "boolean")
-                WebInspector.runtimeManager.evaluateInInspectedWindow("(" + getCompletions + ")(\"" + result.type + "\")", "completion", false, true, true, false, receivedPropertyNamesFromEvaluate.bind(this));
-            else
+            if (result.subtype === "array")
+                result.callFunctionJSON(inspectedPage_evalResult_getArrayCompletions, undefined, receivedArrayPropertyNames.bind(this));
+            else if (result.type === "object" || result.type === "function")
+                result.callFunctionJSON(inspectedPage_evalResult_getCompletions, undefined, receivedPropertyNames.bind(this));
+            else if (result.type === "string" || result.type === "number" || result.type === "boolean" || result.type === "symbol") {
+                let options = {objectGroup: "completion", includeCommandLineAPI: false, doNotPauseOnExceptionsAndMuteConsole: true, returnByValue: false, generatePreview: false, saveResult: false};
+                WebInspector.runtimeManager.evaluateInInspectedWindow("(" + inspectedPage_evalResult_getCompletions + ")(\"" + result.type + "\")", options, receivedPropertyNamesFromEvaluate.bind(this));
+            } else
                 console.error("Unknown result type: " + result.type);
         }
 
@@ -165,6 +199,20 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
             receivedPropertyNames.call(this, result && !wasThrown ? result.value : null);
         }
 
+        function receivedArrayPropertyNames(propertyNames)
+        {
+            // FIXME: <https://webkit.org/b/143589> Web Inspector: Better handling for large collections in Object Trees
+            // If there was an array like object, we generate autocompletion up to 1000 indexes, but this should
+            // handle a list with arbitrary length.
+            if (propertyNames && typeof propertyNames.length === "number") {
+                var max = Math.min(propertyNames.length, 1000);
+                for (var i = 0; i < max; ++i)
+                    propertyNames[i] = true;
+            }
+
+            receivedPropertyNames.call(this, propertyNames);
+        }
+
         function receivedPropertyNames(propertyNames)
         {
             propertyNames = propertyNames || {};
@@ -174,11 +222,15 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
             RuntimeAgent.releaseObjectGroup("completion");
 
             if (!base) {
-                var commandLineAPI = ["$", "$$", "$x", "dir", "dirxml", "keys", "values", "profile", "profileEnd", "monitorEvents", "unmonitorEvents", "inspect", "copy", "clear", "getEventListeners", "$0", "$1", "$2", "$3", "$4", "$_"];
+                var commandLineAPI = ["$", "$$", "$x", "dir", "dirxml", "keys", "values", "profile", "profileEnd", "monitorEvents", "unmonitorEvents", "inspect", "copy", "clear", "getEventListeners", "$0", "$_"];
                 if (WebInspector.debuggerManager.paused && WebInspector.debuggerManager.pauseReason === WebInspector.DebuggerManager.PauseReason.Exception)
                     commandLineAPI.push("$exception");
                 for (var i = 0; i < commandLineAPI.length; ++i)
                     propertyNames[commandLineAPI[i]] = true;
+
+                // FIXME: Due to caching, sometimes old $n values show up as completion results even though they are not available. We should clear that proactively.
+                for (var i = 1; i <= WebInspector.ConsoleCommandResultMessage.maximumSavedResultIndex; ++i)
+                    propertyNames["$" + i] = true;
             }
 
             propertyNames = Object.keys(propertyNames);
@@ -214,10 +266,18 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
             function compare(a, b)
             {
                 // Try to sort in numerical order first.
-                var numericCompareResult = a - b;
+                let numericCompareResult = a - b;
                 if (!isNaN(numericCompareResult))
                     return numericCompareResult;
 
+                // Sort __defineGetter__, __lookupGetter__, and friends last.
+                let aRareProperty = a.startsWith("__") && a.endsWith("__");
+                let bRareProperty = b.startsWith("__") && b.endsWith("__");
+                if (aRareProperty && !bRareProperty)
+                    return 1;
+                if (!aRareProperty && bRareProperty)
+                    return -1;
+
                 // Not numbers, sort as strings.
                 return a.localeCompare(b);
             }
@@ -226,11 +286,11 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
 
             completionController.updateCompletions(completions, implicitSuffix);
         }
-    },
+    }
 
     // Private
 
-    _clearLastProperties: function()
+    _clearLastProperties()
     {
         if (this._clearLastPropertiesTimeout) {
             clearTimeout(this._clearLastPropertiesTimeout);
@@ -242,5 +302,3 @@ WebInspector.JavaScriptRuntimeCompletionProvider.prototype = {
         this._lastPropertyNames = null;
     }
 };
-
-WebInspector.JavaScriptRuntimeCompletionProvider.prototype.__proto__ = WebInspector.Object.prototype;
