@@ -32,15 +32,14 @@
 #include "WebProcess.h"
 #include <WebCore/DisplayRefreshMonitor.h>
 #include <WebCore/TransformationMatrix.h>
-#include <wtf/Functional.h>
 
 // Subclasses
 #if PLATFORM(COCOA)
 #include "RemoteLayerTreeDrawingArea.h"
 #include "TiledCoreAnimationDrawingArea.h"
 #else
-#if USE(COORDINATED_GRAPHICS)
-#include "CoordinatedDrawingArea.h"
+#if USE(COORDINATED_GRAPHICS_MULTIPROCESS)
+#include "AcceleratedDrawingArea.h"
 #else
 #include "DrawingAreaImpl.h"
 #endif
@@ -61,9 +60,9 @@ std::unique_ptr<DrawingArea> DrawingArea::create(WebPage& webPage, const WebPage
     case DrawingAreaTypeRemoteLayerTree:
         return std::make_unique<RemoteLayerTreeDrawingArea>(webPage, parameters);
 #else
-#if USE(COORDINATED_GRAPHICS)
+#if USE(COORDINATED_GRAPHICS_MULTIPROCESS)
     case DrawingAreaTypeCoordinated:
-        return std::make_unique<CoordinatedDrawingArea>(webPage, parameters);
+        return std::make_unique<AcceleratedDrawingArea>(webPage, parameters);
 #else
     case DrawingAreaTypeImpl:
         return std::make_unique<DrawingAreaImpl>(webPage, parameters);
@@ -78,12 +77,12 @@ DrawingArea::DrawingArea(DrawingAreaType type, WebPage& webPage)
     : m_type(type)
     , m_webPage(webPage)
 {
-    WebProcess::shared().addMessageReceiver(Messages::DrawingArea::messageReceiverName(), m_webPage.pageID(), *this);
+    WebProcess::singleton().addMessageReceiver(Messages::DrawingArea::messageReceiverName(), m_webPage.pageID(), *this);
 }
 
 DrawingArea::~DrawingArea()
 {
-    WebProcess::shared().removeMessageReceiver(Messages::DrawingArea::messageReceiverName(), m_webPage.pageID());
+    WebProcess::singleton().removeMessageReceiver(Messages::DrawingArea::messageReceiverName(), m_webPage.pageID());
 }
 
 void DrawingArea::dispatchAfterEnsuringUpdatedScrollPosition(std::function<void ()> function)
@@ -93,7 +92,7 @@ void DrawingArea::dispatchAfterEnsuringUpdatedScrollPosition(std::function<void
 }
 
 #if USE(REQUEST_ANIMATION_FRAME_DISPLAY_MONITOR)
-PassRefPtr<WebCore::DisplayRefreshMonitor> DrawingArea::createDisplayRefreshMonitor(PlatformDisplayID)
+RefPtr<WebCore::DisplayRefreshMonitor> DrawingArea::createDisplayRefreshMonitor(PlatformDisplayID)
 {
     return nullptr;
 }
