@@ -1,4 +1,4 @@
-# Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Apple Inc.  All rights reserved.
+# Copyright (C) 2007-2013, 2015 Apple Inc.  All rights reserved.
 # Copyright (C) 2009, 2010 Chris Jerdonek (chris.jerdonek@gmail.com)
 # Copyright (C) 2010, 2011 Research In Motion Limited. All rights reserved.
 # Copyright (C) 2012 Daniel Bates (dbates@intudata.com)
@@ -60,11 +60,14 @@ BEGIN {
         &exitStatus
         &fixChangeLogPatch
         &gitBranch
+        &gitDirectory
+        &gitTreeDirectory
         &gitdiff2svndiff
         &isGit
         &isGitSVN
         &isGitBranchBuild
         &isGitDirectory
+        &isGitSVNDirectory
         &isSVN
         &isSVNDirectory
         &isSVNVersion16OrNewer
@@ -72,6 +75,7 @@ BEGIN {
         &mergeChangeLogs
         &normalizePath
         &parseChunkRange
+        &parseDiffStartLine
         &parseFirstEOL
         &parsePatch
         &pathRelativeToSVNRepositoryRootForPath
@@ -88,6 +92,7 @@ BEGIN {
         &toWindowsLineEndings
         &gitCommitForSVNRevision
         &listOfChangedFilesBetweenRevisions
+        &unixPath
     );
     %EXPORT_TAGS = ( );
     @EXPORT_OK   = ();
@@ -106,6 +111,7 @@ my $svnVersion;
 # Project time zone for Cupertino, CA, US
 my $changeLogTimeZone = "PST8PDT";
 
+my $unifiedDiffStartRegEx = qr#^--- ([abc]\/)?([^\r\n]+)#;
 my $gitDiffStartRegEx = qr#^diff --git [^\r\n]+#;
 my $gitDiffStartWithPrefixRegEx = qr#^diff --git \w/(.+) \w/([^\r\n]+)#; # We suppose that --src-prefix and --dst-prefix don't contain a non-word character (\W) and end with '/'.
 my $gitDiffStartWithoutPrefixNoSpaceRegEx = qr#^diff --git (\S+) (\S+)$#;
@@ -124,7 +130,7 @@ my $svnPropertyValueNoNewlineRegEx = qr#\ No newline at end of property#;
 sub exitStatus($)
 {
     my ($returnvalue) = @_;
-    if ($^O eq "MSWin32") {
+    if (isWindows()) {
         return $returnvalue >> 8;
     }
     if (!WIFEXITED($returnvalue)) {
@@ -223,15 +229,27 @@ sub isGit()
     return $isGit;
 }
 
-sub isGitSVN()
+sub isGitSVNDirectory($)
 {
-    return $isGitSVN if defined $isGitSVN;
+    my ($directory) = @_;
+
+    my $savedWorkingDirectory = Cwd::getcwd();
+    chdir($directory);
 
     # There doesn't seem to be an officially documented way to determine
     # if you're in a git-svn checkout. The best suggestions seen so far
     # all use something like the following:
     my $output = `git config --get svn-remote.svn.fetch 2>& 1`;
-    $isGitSVN = $output ne '';
+    $isGitSVN = exitStatus($?) == 0 && $output ne "";
+    chdir($savedWorkingDirectory);
+    return $isGitSVN;
+}
+
+sub isGitSVN()
+{
+    return $isGitSVN if defined $isGitSVN;
+
+    $isGitSVN = isGitSVNDirectory(".");
     return $isGitSVN;
 }
 
@@ -319,7 +337,7 @@ sub svnVersion()
 sub isSVNVersion16OrNewer()
 {
     my $version = svnVersion();
-    return eval "v$version" ge v1.6;
+    return "v$version" ge v1.6;
 }
 
 sub chdirReturningRelativePath($)
@@ -465,6 +483,17 @@ sub makeFilePathRelative($)
 }
 
 sub normalizePath($)
+{
+    my ($path) = @_;
+    if (isWindows()) {
+        $path =~ s/\//\\/g;
+    } else {
+        $path =~ s/\\/\//g;
+    }
+    return $path;
+}
+
+sub unixPath($)
 {
     my ($path) = @_;
     $path =~ s/\\/\//g;
@@ -641,6 +670,19 @@ sub isExecutable($)
     return $fileMode % 2;
 }
 
+# Parses an SVN or Git diff header start line.
+#
+# Args:
+#   $line: "Index: " line or "diff --git" line
+#
+# Returns the path of the target file or undef if the $line is unrecognized.
+sub parseDiffStartLine($)
+{
+    my ($line) = @_;
+    return $1 if $line =~ /$svnDiffStartRegEx/;
+    return parseGitDiffStartLine($line) if $line =~ /$gitDiffStartRegEx/;
+}
+
 # Parse the Git diff header start line.
 #
 # Args:
@@ -915,6 +957,103 @@ sub parseSvnDiffHeader($$)
     return (\%header, $_);
 }
 
+# Parse the next Unified diff header from the given file handle, and advance
+# the handle so the last line read is the first line after the header.
+#
+# This subroutine dies if given leading junk.
+#
+# Args:
+#   $fileHandle: advanced so the last line read from the handle is the first
+#                line of the header to parse.  This should be a line
+#                beginning with "Index:".
+#   $line: the line last read from $fileHandle
+#
+# Returns ($headerHashRef, $lastReadLine):
+#   $headerHashRef: a hash reference representing a diff header, as follows--
+#     indexPath: the path of the target file, which is the path found in
+#                the "Index:" line.
+#     isNew: the value 1 if the diff is for a new file.
+#     isDeletion: the value 1 if the diff is a file deletion.
+#     svnConvertedText: the header text converted to a header with the paths
+#                       in some lines corrected.
+#   $lastReadLine: the line last read from $fileHandle.
+sub parseUnifiedDiffHeader($$)
+{
+    my ($fileHandle, $line) = @_;
+
+    $_ = $line;
+
+    my $currentPosition = tell($fileHandle);
+    my $indexLine;
+    my $indexPath;
+    if (/$unifiedDiffStartRegEx/) {
+        # Use $POSTMATCH to preserve the end-of-line character.
+        my $eol = $POSTMATCH;
+        
+        $indexPath = $2;
+
+        # In the case of an addition, we look at the next line for the index path
+        if ($indexPath eq "/dev/null") {
+            $_ = <$fileHandle>;
+            if (/^\+\+\+ ([abc]\/)?([^\t\n\r]+)/) {
+                $indexPath = $2;
+            } else {
+                die "Unrecognized unified diff format.";
+            }
+            $_ = $line;
+        }
+
+        $indexLine = "Index: $indexPath$eol"; # Convert to SVN format.
+    } else {
+        die("Could not parse leading \"---\" line: \"$line\".");
+    }
+
+    seek($fileHandle, $currentPosition, 0);
+
+    my $isDeletion;
+    my $isHeaderEnding;
+    my $isNew;
+    my $svnConvertedText = $indexLine;
+    while (1) {
+        # Temporarily strip off any end-of-line characters to simplify
+        # regex matching below.
+        s/([\n\r]+)$//;
+        my $eol = $1;
+        
+        if (/^--- \/dev\/null/) {
+            $isNew = 1;
+        } elsif (/^\+\+\+ \/dev\/null/) {
+            $isDeletion = 1;
+        }
+        
+        if (/^(---|\+\+\+) ([abc]\/)?([^\t\n\r]+)/) {
+            if ($1 eq "---") {
+                my $prependText = "";
+                $prependText = "new file mode 100644\n" if $isNew;
+                $_ = "${prependText}index 0000000..0000000\n$1 $3";
+            } else {
+                $_ = "$1 $3";
+                $isHeaderEnding = 1;
+            }
+        }
+        
+        $svnConvertedText .= "$_$eol"; # Also restore end-of-line characters.
+        
+        $currentPosition = tell($fileHandle);
+        $_ = <$fileHandle>; # Not defined if end-of-file reached.
+        last if (!defined($_) || /$unifiedDiffStartRegEx/ || $isHeaderEnding);
+    }
+    
+    my %header;
+    
+    $header{indexPath} = $indexPath;
+    $header{isDeletion} = $isDeletion if $isDeletion;
+    $header{isNew} = $isNew if $isNew;
+    $header{svnConvertedText} = $svnConvertedText;
+
+    return (\%header, $_);
+}
+
 # Parse the next diff header from the given file handle, and advance
 # the handle so the last line read is the first line after the header.
 #
@@ -953,6 +1092,7 @@ sub parseDiffHeader($$)
     my $header;  # This is a hash ref.
     my $isGit;
     my $isSvn;
+    my $isUnified;
     my $lastReadLine;
 
     if ($line =~ $svnDiffStartRegEx) {
@@ -961,12 +1101,16 @@ sub parseDiffHeader($$)
     } elsif ($line =~ $gitDiffStartRegEx) {
         $isGit = 1;
         ($header, $lastReadLine) = parseGitDiffHeader($fileHandle, $line);
+    } elsif ($line =~ $unifiedDiffStartRegEx) {
+        $isUnified = 1;
+        ($header, $lastReadLine) = parseUnifiedDiffHeader($fileHandle, $line);
     } else {
         die("First line of diff does not begin with \"Index:\" or \"diff --git\": \"$line\"");
     }
 
     $header->{isGit} = $isGit if $isGit;
     $header->{isSvn} = $isSvn if $isSvn;
+    $header->{isUnified} = $isUnified if $isUnified;
 
     return ($header, $lastReadLine);
 }
@@ -1051,6 +1195,10 @@ sub parseDiff($$;$)
             # all diffs in the patch are formatted the same (SVN or Git).
             $headerStartRegEx = $gitDiffStartRegEx;
         }
+        
+        if (!$headerHashRef && ($line =~ $unifiedDiffStartRegEx)) {
+            $headerStartRegEx = $unifiedDiffStartRegEx;
+        }
 
         if ($line =~ $svnPropertiesStartRegEx) {
             my $propertyPath = $1;
@@ -1909,7 +2057,10 @@ sub changeLogNameError($)
 
 sub changeLogName()
 {
-    my $name = $ENV{CHANGE_LOG_NAME} || $ENV{REAL_NAME} || gitConfig("user.name") || (split /\s*,\s*/, (getpwuid $<)[6])[0];
+    my $name = $ENV{CHANGE_LOG_NAME} || $ENV{REAL_NAME} || gitConfig("user.name");
+    if (not $name and !isWindows()) {
+        $name = (split /\s*,\s*/, (getpwuid $<)[6])[0];
+    }
 
     changeLogNameError("Failed to determine ChangeLog name.") unless $name;
     # getpwuid seems to always succeed on windows, returning the username instead of the full name.  This check will catch that case.
@@ -2010,8 +2161,8 @@ sub decodeGitBinaryPatch($$)
     # Then, content of the chunk comes. To decode the content, we
     # need decode it with base85 first, and then zlib.
     my $gitPatchRegExp = '(literal|delta) ([0-9]+)\n([A-Za-z0-9!#$%&()*+-;<=>?@^_`{|}~\\n]*?)\n\n';
-    if ($contents !~ m"\nGIT binary patch\n$gitPatchRegExp$gitPatchRegExp\Z") {
-        die "$fullPath: unknown git binary patch format"
+    if ($contents !~ m"\nGIT binary patch\n$gitPatchRegExp$gitPatchRegExp(\Z|-- \n)") {
+        return ();
     }
 
     my $binaryChunkType = $1;
