@@ -44,19 +44,28 @@ TEST(WebCore, UserAgentQuirksTest)
     EXPECT_TRUE(uaString.contains("Mac OS X"));
     EXPECT_FALSE(uaString.contains("Linux"));
     EXPECT_FALSE(uaString.contains("Chrome"));
+    EXPECT_FALSE(uaString.contains("Firefox"));
 
-    // google domains require Chrome in the UA
+    // google domains require Firefox in the UA
     uaString = standardUserAgentForURL(URL(ParsedURLString, "http://www.google.es/"));
+    EXPECT_FALSE(uaString.contains("Chrome"));
+    EXPECT_FALSE(uaString.contains("Safari"));
+    EXPECT_FALSE(uaString.contains("Chromium"));
+    EXPECT_FALSE(uaString.contains("AppleWebKit"));
+    EXPECT_TRUE(uaString.contains("Firefox"));
     EXPECT_FALSE(uaString.contains("Macintosh"));
     EXPECT_FALSE(uaString.contains("Mac OS X"));
-    EXPECT_TRUE(uaString.contains("Linux"));
-    EXPECT_TRUE(uaString.contains("Chrome"));
+    EXPECT_FALSE(uaString.contains("Windows"));
 
     uaString = standardUserAgentForURL(URL(ParsedURLString, "http://maps.google.com/"));
+    EXPECT_FALSE(uaString.contains("Chrome"));
+    EXPECT_FALSE(uaString.contains("Safari"));
+    EXPECT_FALSE(uaString.contains("Chromium"));
+    EXPECT_FALSE(uaString.contains("AppleWebKit"));
+    EXPECT_TRUE(uaString.contains("Firefox"));
     EXPECT_FALSE(uaString.contains("Macintosh"));
     EXPECT_FALSE(uaString.contains("Mac OS X"));
-    EXPECT_TRUE(uaString.contains("Linux"));
-    EXPECT_TRUE(uaString.contains("Chrome"));
+    EXPECT_FALSE(uaString.contains("Windows"));
 
     // Slack requires Chrome in the UA
     uaString = standardUserAgentForURL(URL(ParsedURLString, "http://www.slack.com/"));
@@ -64,6 +73,7 @@ TEST(WebCore, UserAgentQuirksTest)
     EXPECT_FALSE(uaString.contains("Mac OS X"));
     EXPECT_TRUE(uaString.contains("Linux"));
     EXPECT_TRUE(uaString.contains("Chrome"));
+    EXPECT_FALSE(uaString.contains("Firefox"));
 }
 
 } // namespace TestWebKitAPI
